package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.entry.BpmOAEntryCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.entry.BpmOAEntryRespVO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAEntryConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAEntryDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAEntryService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 入职申请 Controller
 *
 * @author 符溶馨
 */
@Tag(name = "管理后台 - OA 入职申请")
@RestController
@RequestMapping("/bpm/oa/entry")
@Validated
public class BpmOAEntryController {

    @Resource
    private BpmOAEntryService entryService;

    @Resource
    private FileApi fileApi;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    @PermitAll
    public CommonResult<Long> createEntry(@Valid @RequestBody BpmOAEntryCreateReqVO createReqVO) {

        return success(entryService.createEntry(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得入职申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAEntryRespVO> getEntry(@RequestParam("id") Long id) {

        BpmOAEntryDO entry = entryService.getEntry(id);

        return success(BpmOAEntryConvert.INSTANCE.convert(entry));
    }

    @GetMapping("/getByOpenId")
    @Operation(summary = "获得入职申请")
    @Parameter(name = "openId", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAEntryRespVO> getEntryByOpenId(@RequestParam("openId") String openId) {

        BpmOAEntryDO entry = entryService.getEntryByOpenId(openId);

        return success(BpmOAEntryConvert.INSTANCE.convert(entry));
    }
}
