package cn.iocoder.yudao.module.system.service.attendance;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.BetweenFormatter;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.Constants;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.distance.GeoUtil;
import cn.iocoder.yudao.module.system.controller.admin.attendance.dto.*;
import cn.iocoder.yudao.module.system.controller.admin.attendance.vo.*;
import cn.iocoder.yudao.module.system.controller.admin.punchrecord.vo.AttendancePunchRecordSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshift.AttendanceGroupShiftDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.groupshiftitem.AttendanceGroupShiftItemDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.punchrecord.AttendancePunchRecordDO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.group.AttendanceGroupMapper;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.punchrecord.AttendancePunchRecordMapper;
import cn.iocoder.yudao.module.system.handler.PunchHandler;
import cn.iocoder.yudao.module.system.service.attendance.group.AttendanceGroupService;
import cn.iocoder.yudao.module.system.service.attendance.groupshift.AttendanceGroupShiftService;
import cn.iocoder.yudao.module.system.service.attendance.groupshiftitem.AttendanceGroupShiftItemService;
import cn.iocoder.yudao.module.system.service.attendance.groupuser.AttendanceGroupUserService;
import cn.iocoder.yudao.module.system.service.attendance.punch.PunchService;
import cn.iocoder.yudao.module.system.service.attendance.punch.dto.AttendanceOnTheDayDTO;
import cn.iocoder.yudao.module.system.service.attendance.punchrecord.AttendancePunchRecordService;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.hutool.core.date.DateUtil.dayOfWeekEnum;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 考勤 实现类
 */
@Service
@Slf4j
public class AttendanceServiceImpl implements AttendanceService {
    @Resource
    private PunchHandler punchHandler;
    @Resource
    private AttendanceGroupService attendanceGroupService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;
    @Resource
    private AttendanceGroupShiftService attendanceGroupShiftService;
    @Resource
    private AttendanceGroupShiftItemService attendanceGroupShiftItemService;
    @Resource
    private AttendancePunchRecordService attendancePunchRecordService;
    @Resource
    private AttendancePunchRecordMapper attendancePunchRecordMapper;
    @Resource
    private AdminUserService adminUserService;

    @Resource
    private AttendanceGroupMapper attendanceGroupMapper;
    @Resource
    private AttendanceGroupUserService attendanceGroupUserService;


    // 定义一些常量以提高代码的可读性和可维护性
    /**
     * 打卡类型(0上班卡 1下班卡 2迟到卡 3早退卡 4未到打卡时间)
     */
    private static final Integer PUNCH_TYPE_UP_WORK = 0;
    private static final Integer PUNCH_TYPE_DOWN_WORK = 1;
    private static final Integer PUNCH_TYPE_BE_LATE = 2;
    private static final Integer PUNCH_TYPE_LEAVE_EARLY = 3;
    private static final Integer PUNCH_TYPE_NOT_COME_YET = 4;

    /**
     * 上下班标识 0上班 1下班
     */
    private static final Integer UP_WORK = 0;
    private static final Integer DOWN_WORK = 1;

    @Override
    public AttendancePunchPageVO getPunchPage(AttendancePunchPageDTO dto) {
        // -- 获取当前登陆用户 - 如果当前登陆用户状态异常 - 则不允许打卡
        AdminUserDO user = adminUserService.getUser(dto.getUserId());
        dto.setUser(user);
        AttendancePunchPageVO vo = new AttendancePunchPageVO();
        //获取当前登录用户所在群组
        AttendanceGroupDO activationGroup = attendanceGroupService.getByUserId(dto.getUserId());
        //不在考勤组 - 返回回去
        if (activationGroup == null) {
            vo.setInGroup(Constants.FALSE);
            return vo;
        }
        // 策略模式 - 将不同考勤类型的 分散开逻辑 - 后续好拓展
        PunchService punchService = punchHandler.getResource(AttendanceGroupDO.getCodeByType(activationGroup.getType()));
        return punchService.getPunchPage(dto.setActivationGroup(activationGroup));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public AttendancePunchVO punch(AttendancePunchDTO dto) {
        AttendancePunchPageVO pageVO = this.getPunchPage(new AttendancePunchPageDTO().setPunchType(dto.getPunchType())
                .setLongitude(dto.getLongitude()).setLatitude(dto.getLatitude())
                .setFlag(true).setLocalDateTime(dto.getLocalDateTime()).setUserId(dto.getUserId()));
        if (pageVO.getUser() == null || pageVO.getUser().getStatus().equals(CommonStatusEnum.DISABLE.getStatus())) {
            throw exception(USER_NOT_EXISTS);
        }
        // -- 获取当天的考勤记录
        if (Constants.FALSE.equals(pageVO.getInGroup())) {
            throw exception(NOT_IN_THE_ATTENDANCE_GROUP);
        }
        if (Constants.FALSE.equals(pageVO.getTodayNeedAttendance())) {
            throw exception(NO_ATTENDANCE_REQUIRED_ON_THE_DAY);
        }
        if (Constants.FALSE.equals(pageVO.getFieldworkFlag()) && Constants.FALSE.equals(pageVO.getPunchPoint())) {
            throw exception(DID_NOT_ENTER_THE_ATTENDANCE_RANGE);
        }
        if (PUNCH_TYPE_NOT_COME_YET.equals(pageVO.getPunchType())) {
            throw exception(ATTENDANCE_TIME_NOT_ARRIVED);
        }
        AttendancePunchVO vo = new AttendancePunchVO();
        // -- 插入数据到数据库中 - 如果插入成功 更新redis
        List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = pageVO.getAttendanceOnTheDayDTOS();
        AttendanceOnTheDayDTO dayDTO = attendanceOnTheDayDTOS.get(pageVO.getIndex());
        Integer status = AttendancePunchRecordSaveReqVO.convertStatus(pageVO.getPunchType());


        //更新预打卡记录 - update 如果修改数量为0 则新增
        AttendancePunchRecordSaveReqVO attendancePunchRecordSaveReqVO = new AttendancePunchRecordSaveReqVO()
                .setUserId(dto.getUserId())
                .setDeptId(pageVO.getUser().getDeptId())
                .setAttendanceGroupId(pageVO.getActivationGroup().getId())
                .setAttendanceGroupName(pageVO.getActivationGroup().getGroupName())
                .setAttendanceGroupShiftId(pageVO.getAttendanceGroupShiftDO().getId())
                .setAttendanceGroupShiftName(pageVO.getAttendanceGroupShiftDO().getName())
                .setAttendanceGroupShiftItemId(pageVO.getAttendanceGroupShiftItemId())
                .setType(pageVO.getActivationGroup().getType())
                .setPunchType(dto.getPunchType())
                .setWorkType(pageVO.getType())
                .setLevel(dayDTO.getLevel())
                .setStatus(status)
                .setNextDayFlag(Constants.FALSE)
                .setFieldServiceFlag(
                        (Constants.TRUE.equals(pageVO.getFieldworkFlag()) && Constants.FALSE.equals(pageVO.getPunchPoint()))
                                ? Constants.TRUE : Constants.FALSE)
                .setDayTime(pageVO.getTargetDayStr())
                .setShouldPunchTime(pageVO.getShouldPunchTime())
                .setPunchTime(dto.getLocalDateTime())
                //取绝对值 如果status是负数则变为正数
                .setLateTime(Constants.ONE.equals(status) ? Math.abs(LocalDateTimeUtil.between(dto.getLocalDateTime(), pageVO.getShouldPunchTime(), ChronoUnit.MILLIS)) : 0L)
                .setLeaveEarlyTime(Constants.TWO.equals(status) ? Math.abs(LocalDateTimeUtil.between(dto.getLocalDateTime(), pageVO.getShouldPunchTime(), ChronoUnit.MILLIS)) : 0L)
                .setWorkOvertimeTime(Constants.ZERO.equals(status) && Constants.ONE.equals(pageVO.getPunchType()) ? Math.abs(LocalDateTimeUtil.between(dto.getLocalDateTime(), pageVO.getShouldPunchTime(), ChronoUnit.MILLIS)) : 0L)
                .setRemark(dto.getRemark())
                .setImage(dto.getImage())
                .setPunchAddress(dto.getPunchAddress());
        AttendancePunchRecordDO attendancePunchRecordDO = new AttendancePunchRecordDO();
        BeanUtil.copyProperties(attendancePunchRecordSaveReqVO, attendancePunchRecordDO);

        int updateNum = attendancePunchRecordMapper.update(attendancePunchRecordDO, new LambdaUpdateWrapper<AttendancePunchRecordDO>()
                .eq(AttendancePunchRecordDO::getUserId, dto.getUserId())
                .eq(AttendancePunchRecordDO::getDayTime, pageVO.getTargetDayStr())
                .eq(AttendancePunchRecordDO::getNextDayFlag, Constants.TRUE)
                .eq(AttendancePunchRecordDO::getStatus, AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH)
                .eq(AttendancePunchRecordDO::getWorkType, attendancePunchRecordSaveReqVO.getWorkType())
                .eq(AttendancePunchRecordDO::getAttendanceGroupShiftItemId, attendancePunchRecordSaveReqVO.getAttendanceGroupShiftItemId())
        );
        Long id = null;
        if (updateNum <= 0) {
            id = attendancePunchRecordService.createPunchRecord(attendancePunchRecordSaveReqVO);
        }
        // -- 新增成功后更新redis
        if (updateNum > 0 || id != null) {
            dayDTO.setPunchTime(dto.getLocalDateTime().format(Constants.REPO_DATE_FORMAT));
            dayDTO.setPunchStatus(attendancePunchRecordSaveReqVO.getStatus());
            dayDTO.setFieldServiceFlag(attendancePunchRecordSaveReqVO.getFieldServiceFlag());
            dayDTO.setPunchAddress(dto.getPunchAddress());
            dayDTO.setRemark(dto.getRemark());
            dayDTO.setImage(dto.getImage());
            dayDTO.setShouldPunchTime(pageVO.getShouldPunchTime());
            dayDTO.setPunchLocalDateTime(dto.getLocalDateTime());
            dayDTO.setLateTime(attendancePunchRecordSaveReqVO.getLateTime());
            dayDTO.setLeaveEarlyTime(attendancePunchRecordSaveReqVO.getLeaveEarlyTime());

            String mapKey = dto.getUserId().toString();
            stringRedisTemplate.opsForHash().put(pageVO.getRedisKey(), mapKey, JSONUtil.toJsonStr(attendanceOnTheDayDTOS));
        }
        vo.setStatus(status);
        vo.setActivationGroup(pageVO.getActivationGroup());
        vo.setAttendanceGroupShiftDO(pageVO.getAttendanceGroupShiftDO());
        vo.setAttendanceOnTheDayDTOS(attendanceOnTheDayDTOS);
        return vo;
    }


    @Override
    public Boolean isHoliday(LocalDateTime localDateTime) {
        String dateStr = localDateTime.format(Constants.REPO_DATE_FORMAT);
        String year = dateStr.substring(0, 4);
        String key = "holiday_" + year;
        Boolean flag = stringRedisTemplate.hasKey(key);
        // 缓存不存在
        if (Boolean.FALSE.equals(flag)) {
            String url = "https://timor.tech/api/holiday/year/" + year;
            String json = HttpUtil.get(url, 3000);
            Map<String, String> map = new HashMap<>();
            JSONObject jsonObject = JSONUtil.parseObj(json);
            Object o = jsonObject.get("holiday");
            //将o转为json对象并且循环获取对象中所有的属性
            JSONObject data = JSONUtil.parseObj(o);
            data.forEach((k, v) -> {
                JSONObject item = JSONUtil.parseObj(v);
                map.put(item.get("date").toString(), item.get("holiday").toString());
            });
            stringRedisTemplate.opsForHash().putAll(key, map);
            // -- 删除去年的
            String lastKey = "holiday_" + (Integer.parseInt(year) - 1);
            stringRedisTemplate.delete(lastKey);
        }
        return (Boolean) stringRedisTemplate.opsForHash().get(key, dateStr);
    }


    @Override
    public void calculatePunch(AttendancePunchPageDTO dto, AttendancePunchPageVO vo) {
        LocalDateTime localDateTime = dto.getLocalDateTime();
        LocalDateTime nextDayDateTime = LocalDateTimeUtil.offset(localDateTime, 1, ChronoUnit.DAYS);
        AttendanceGroupDO activationGroup = dto.getActivationGroup();
        vo.setFieldworkFlag(activationGroup.getFieldworkFlag());
        // - 根据经纬度判断是否在对应班组的打卡点上 - 如果是考勤机的话默认就是在打卡点
        vo.setPunchPoint(AttendanceGroupDO.PUNCH_TYPE_ATTENDANCE_MACHINE.equals(dto.getPunchType()) ? Constants.TRUE :
                GeoUtil.distance(Double.parseDouble(dto.getLatitude()), Double.parseDouble(dto.getLongitude())
                        , Double.parseDouble(activationGroup.getLatitude()), Double.parseDouble(activationGroup.getLongitude()), activationGroup.getScope()));

        // -- 获取班次
        AttendanceGroupShiftDO attendanceGroupShiftDO = attendanceGroupShiftService.getGroupShift(vo.getAttendanceGroupShiftId());
        // -- 获取班次item
        List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOList = attendanceGroupShiftItemService.getGroupShiftItemListByShiftId(vo.getAttendanceGroupShiftId());

        String key = Constants.ATTENDANCE + Constants.UNDERLINE + activationGroup.getId() + Constants.UNDERLINE; // + 时间
        String mapKey = dto.getUserId().toString();


        String yesterdayStr = LocalDateTimeUtil.offset(localDateTime, -1, ChronoUnit.DAYS).format(Constants.REPO_DATE_FORMAT);
        String targetDayStr = localDateTime.format(Constants.REPO_DATE_FORMAT);
        AttendanceGroupShiftItemDO last = CollUtil.getLast(attendanceGroupShiftItemDOList);
        // -- 如果最后的时间是跨天 那么 需要判断当前时间是否在跨天时间段内 - 如果是的话取昨天的的yyyy-MM-dd
        if (Arrays.asList(1, 2).contains(last.getNextDayFlag())) {
            LocalDateTime endOfDay = LocalDateTime.ofInstant(DateUtils.buildHHmmTime(last.getEndTime()).toInstant(), ZoneId.systemDefault()).plusMinutes(last.getAfterPunchTimeDownWork());
            LocalDateTime beginOfDay = LocalDateTimeUtil.beginOfDay(endOfDay);
            if (LocalDateTimeUtil.isIn(localDateTime, beginOfDay, endOfDay)) {
                targetDayStr = yesterdayStr;
            }
        }

        List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = this.getAttendanceOnTheDay(key, mapKey, targetDayStr, attendanceGroupShiftItemDOList);
        // -- 这里没有的情况 只可能是redis 数据库崩了/或者班次子表没数据 - 暂时先返回不需要考勤
        if (attendanceOnTheDayDTOS.isEmpty()) {
            vo.setTodayNeedAttendance(Constants.FALSE);
            return;
        }

        // 定义是否需要更新redis - 默认不需要更新
        boolean updateRedis = false;
        int index = 0;
        for (AttendanceOnTheDayDTO attendanceOnTheDayDTO : attendanceOnTheDayDTOS) {
            if (AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH.equals(attendanceOnTheDayDTO.getPunchStatus())) {
                LocalDateTime time = LocalDateTime.ofInstant(DateUtils.buildHHmmTime(attendanceOnTheDayDTO.getTime(), (attendanceOnTheDayDTO.getNextDayFlag() == 0 ? localDateTime : nextDayDateTime)).toInstant(), ZoneId.systemDefault());
                LocalDateTime lastTime = time.minusMinutes(attendanceOnTheDayDTO.getBeforePunchTime());
                LocalDateTime endTime = time.plusMinutes(attendanceOnTheDayDTO.getAfterPunchTime());
                // 确保beforePunchTime/AfterPunchTime是正数 - 判断当前时间与最早打卡时间 如果没到时间则跳过
                if (attendanceOnTheDayDTO.getBeforePunchTime() < Constants.ZERO || attendanceOnTheDayDTO.getAfterPunchTime() < Constants.ZERO || localDateTime.isBefore(lastTime)) {
                    continue; // 跳过当前循环，避免错误 - 如果所有的都错了话 那么返回默认的 未到打卡时间
                }
                if (LocalDateTimeUtil.isIn(localDateTime, lastTime, endTime)) {
                    int punchType = this.determinePunchType(localDateTime, time, UP_WORK.equals(attendanceOnTheDayDTO.getType()));
                    vo.setPunchType(punchType);
                    if (dto.getFlag()) {
                        vo.setType(attendanceOnTheDayDTO.getType());
                        vo.setShouldPunchTime(time);
                        vo.setAttendanceGroupShiftItemId(attendanceOnTheDayDTO.getId());
                        vo.setIndex(index);
                    }
                    break;
                } else {
                    // 如果是未打卡 - 判断当前时间是否在打卡区间内 - 如果不在的话直接set缺卡
                    // - 定时任务还没来得及刷新redis 数据 - 这里需要手动刷新
                    updateRedis = true;
                    attendanceOnTheDayDTO.setPunchStatus(AttendanceOnTheDayDTO.PUNCH_STATUS_MISS);
                }
            }
            index++;
        }
        vo.setTargetDayStr(targetDayStr);
        vo.setAttendanceOnTheDayDTOS(attendanceOnTheDayDTOS);
        if (dto.getFlag()) {
            vo.setRedisKey(key + targetDayStr);
            vo.setAttendanceGroupShiftDO(attendanceGroupShiftDO);
        }
        // 更新redis
        if (updateRedis) {
            stringRedisTemplate.opsForHash().put(key + targetDayStr, mapKey, JSONUtil.toJsonStr(attendanceOnTheDayDTOS));
        }
    }


    /**
     * 确定打卡类型 - 上班打卡 忽略秒 比如 8点上班 8点0分59秒打卡不算迟到
     *
     * @param localDateTime
     * @param time
     * @param isUpWork
     * @return
     */
    private int determinePunchType(LocalDateTime localDateTime, LocalDateTime time, boolean isUpWork) {
        if (localDateTime.isAfter(time.plusMinutes(Constants.ONE)) && isUpWork) {
            return PUNCH_TYPE_BE_LATE;
        } else if (time.plusMinutes(1).isAfter(localDateTime) && isUpWork) {
            return PUNCH_TYPE_UP_WORK;
        } else if (localDateTime.isAfter(time) && !isUpWork) {
            return PUNCH_TYPE_DOWN_WORK;
        } else {
            return PUNCH_TYPE_LEAVE_EARLY;
        }
    }

    private List<AttendanceOnTheDayDTO> getAttendanceOnTheDay(String key, String mapKey, String targetDayStr, List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOList) {
        List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = new ArrayList<>();
        // 尝试从Redis获取数据
        try {
            attendanceOnTheDayDTOS = this.loadFromRedis(key, targetDayStr, mapKey);
        } catch (Exception e) {
            // Redis操作异常处理逻辑
            log.error("Load from Redis failed.", e);
        }

        // 如果Redis中没有，则从数据库加载并存入Redis
        if (attendanceOnTheDayDTOS.isEmpty()) {
            attendanceOnTheDayDTOS = this.buildAttendanceOnTheDay(attendanceGroupShiftItemDOList);
            this.saveToRedis(key, targetDayStr, mapKey, attendanceOnTheDayDTOS);
        }
        return attendanceOnTheDayDTOS;
    }

    private List<AttendanceOnTheDayDTO> loadFromRedis(String key, String targetDayStr, String mapKey) {
        Object o = stringRedisTemplate.opsForHash().get(key + targetDayStr, mapKey);
        return o != null ? JSONUtil.toList(o.toString(), AttendanceOnTheDayDTO.class) : new ArrayList<>();
    }

    /**
     * 考勤数据设置两天过期
     *
     * @param key
     * @param targetDayStr
     * @param mapKey
     * @param attendanceOnTheDayDTOS
     */
    private void saveToRedis(String key, String targetDayStr, String mapKey, List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS) {
        stringRedisTemplate.opsForHash().put(key + targetDayStr, mapKey, JSONUtil.toJsonStr(attendanceOnTheDayDTOS));
        //设置缓存 2天
        stringRedisTemplate.expire(key + targetDayStr, 2, TimeUnit.DAYS);
    }

    @Override
    public List<AttendanceOnTheDayDTO> buildAttendanceOnTheDay(List<AttendanceGroupShiftItemDO> attendanceGroupShiftItemDOList) {
        List<AttendanceOnTheDayDTO> attendanceOnTheDayDTOS = new ArrayList<>();
        //上下班时间
        for (AttendanceGroupShiftItemDO attendanceGroupShiftItemDO : attendanceGroupShiftItemDOList) {
            AttendanceOnTheDayDTO dto = new AttendanceOnTheDayDTO();
            dto.setId(attendanceGroupShiftItemDO.getId());
            dto.setKqAttendanceGroupShiftId(attendanceGroupShiftItemDO.getKqAttendanceGroupShiftId());
            dto.setKqAttendanceGroupShiftName(attendanceGroupShiftItemDO.getKqAttendanceGroupShiftName());
            dto.setTime(attendanceGroupShiftItemDO.getBeginTime());
            dto.setPunchTime(StringUtils.EMPTY);
            dto.setType(UP_WORK);
            dto.setLevel(attendanceGroupShiftItemDO.getLevel());
            dto.setBeforePunchTime(attendanceGroupShiftItemDO.getBeforePunchTimeUpWork());
            dto.setAfterPunchTime(attendanceGroupShiftItemDO.getAfterPunchTimeUpWork());
            dto.setPunchStatus(AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH);
            dto.setFieldServiceFlag(Constants.FALSE);
            dto.setNextDayFlag(attendanceGroupShiftItemDO.getNextDayFlag() == 0 || attendanceGroupShiftItemDO.getNextDayFlag() == 2 ? 0 : 1);
            attendanceOnTheDayDTOS.add(dto);

            dto = new AttendanceOnTheDayDTO();
            dto.setId(attendanceGroupShiftItemDO.getId());
            dto.setKqAttendanceGroupShiftId(attendanceGroupShiftItemDO.getKqAttendanceGroupShiftId());
            dto.setKqAttendanceGroupShiftName(attendanceGroupShiftItemDO.getKqAttendanceGroupShiftName());
            dto.setTime(attendanceGroupShiftItemDO.getEndTime());
            dto.setPunchTime(StringUtils.EMPTY);
            dto.setType(DOWN_WORK);
            dto.setLevel(attendanceGroupShiftItemDO.getLevel());
            dto.setBeforePunchTime(attendanceGroupShiftItemDO.getBeforePunchTimeDownWork());
            dto.setAfterPunchTime(attendanceGroupShiftItemDO.getAfterPunchTimeDownWork());
            dto.setPunchStatus(AttendanceOnTheDayDTO.PUNCH_STATUS_UN_PUNCH);
            dto.setFieldServiceFlag(Constants.FALSE);
            dto.setNextDayFlag(attendanceGroupShiftItemDO.getNextDayFlag() == 0 ? 0 : 1);
            attendanceOnTheDayDTOS.add(dto);
        }
        return attendanceOnTheDayDTOS;
    }

    @Override
    public Map<String, AttendanceStatusByDayVO> statisticsByDay(AttendanceStatisticsByDayDTO dto) {
        //获取当前天 所在月份所有日期
        List<String> dateList = DateUtils.betweenDayList(DateUtil.beginOfMonth(dto.getTime()),
                DateUtil.endOfMonth(dto.getTime()));
        List<AttendancePunchRecordDO> list = attendancePunchRecordMapper.selectList(new LambdaQueryWrapper<AttendancePunchRecordDO>()
                .eq(AttendancePunchRecordDO::getUserId, dto.getUserId())
                .in(AttendancePunchRecordDO::getDayTime, dateList)
                .orderByAsc(AttendancePunchRecordDO::getDayTime)
                .orderByAsc(AttendancePunchRecordDO::getLevel)
                .orderByAsc(AttendancePunchRecordDO::getWorkType));
        Map<String, AttendanceStatusByDayVO> map = new HashMap<>();

        Map<String, List<AttendancePunchRecordDO>> collect = list.stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getDayTime));

        List<Integer> statusList = Arrays.asList(AttendanceOnTheDayDTO.PUNCH_STATUS_LATE, AttendanceOnTheDayDTO.PUNCH_STATUS_LEAVE_EARLY, AttendanceOnTheDayDTO.PUNCH_STATUS_MISS);
        for (Map.Entry<String, List<AttendancePunchRecordDO>> entry : collect.entrySet()) {
            AttendanceStatusByDayVO attendanceStatusByDayVO = new AttendanceStatusByDayVO();
            int status = 0;
            for (AttendancePunchRecordDO attendancePunchRecordDO : entry.getValue()) {
                if (statusList.contains(attendancePunchRecordDO.getStatus())) {
                    status = 1;
                    break;
                }
            }
            attendanceStatusByDayVO.setStatus(status);
            attendanceStatusByDayVO.setSubmitStatus(Constants.FALSE);
            List<AttendanceByGroupVO> attendanceByGroupVOS = new ArrayList<>();
            Map<String, List<AttendancePunchRecordDO>> itemMap = entry.getValue().stream().collect(Collectors.groupingBy(a -> a.getAttendanceGroupId() + "_" + a.getAttendanceGroupShiftId()));
            for (Map.Entry<String, List<AttendancePunchRecordDO>> stringListEntry : itemMap.entrySet()) {
                if (CollectionUtil.isNotEmpty(stringListEntry.getValue())) {
                    AttendancePunchRecordDO attendancePunchRecordDO = stringListEntry.getValue().get(0);
                    AttendanceByGroupVO attendanceByGroupVO = new AttendanceByGroupVO();
                    attendanceByGroupVO.setGroupName(attendancePunchRecordDO.getAttendanceGroupName());
                    attendanceByGroupVO.setAttendanceGroupShiftName(attendancePunchRecordDO.getAttendanceGroupShiftName());
                    attendanceByGroupVO.setList(stringListEntry.getValue());
                    attendanceByGroupVOS.add(attendanceByGroupVO);
                }
            }
            attendanceStatusByDayVO.setList(attendanceByGroupVOS);
            map.put(entry.getKey(), attendanceStatusByDayVO);
        }
        return map;
    }

    @Override
    public AttendanceStatisticsVO statisticsByCycle(AttendanceStatisticsByCycleDTO dto) {
        AttendanceStatisticsVO vo = new AttendanceStatisticsVO();
        Date thisTime = new Date();
        Date beginTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        if (beginTime.getTime() > endTime.getTime()) {
            throw exception(THE_START_TIME_CANNOT_BE_GREATER_THAN_THE_END_TIME);
        }
        // -- 如果结束时间大于当前时间 - 那么结束时间 = 当前时间
        if (endTime.getTime() > thisTime.getTime()) {
            endTime = thisTime;
        }
        List<String> dateList = DateUtils.betweenDayList(beginTime, endTime);
        List<AttendancePunchRecordDO> list = attendancePunchRecordMapper.selectList(new LambdaQueryWrapper<AttendancePunchRecordDO>()
                .eq(AttendancePunchRecordDO::getUserId, dto.getUserId())
                .in(AttendancePunchRecordDO::getDayTime, dateList)
                .eq(AttendancePunchRecordDO::getNextDayFlag, Constants.FALSE)
                .orderByAsc(AttendancePunchRecordDO::getCreateTime)
                .orderByAsc(AttendancePunchRecordDO::getWorkType)
                .orderByAsc(AttendancePunchRecordDO::getLevel));
        // ---
        Map<String, List<AttendancePunchRecordDO>> map = list.stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getDayTime, TreeMap::new, Collectors.toList()));
        List<AttendancePunchStatisticsVO> averageWorkingHours = new ArrayList<>();
        List<AttendancePunchStatisticsVO> attendanceDays = new ArrayList<>();
        List<AttendancePunchStatisticsVO> beLateNumber = new ArrayList<>();
        List<AttendancePunchStatisticsVO> earlyDeparturesNumber = new ArrayList<>();
        List<AttendancePunchStatisticsVO> missingCardsNumber = new ArrayList<>();
        List<AttendancePunchStatisticsVO> minerDays = new ArrayList<>();
        List<AttendancePunchStatisticsVO> fieldServiceNumber = new ArrayList<>();
        CalculateNum calculateNum = new CalculateNum();

        for (Map.Entry<String, List<AttendancePunchRecordDO>> entry : map.entrySet()) {
            String day = entry.getKey();
            String weekChinese = dayOfWeekEnum(DateUtil.parse(entry.getKey())).toChinese();
            // -- 按照班次子表分组
            Map<Long, List<AttendancePunchRecordDO>> workMap = entry.getValue().stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getAttendanceGroupShiftItemId));
            // -- 出勤天数计算
            AttendancePunchStatisticsVO attendanceDayVO = this.calculateAttendanceDays(day, weekChinese, entry.getValue(), calculateNum);
            if (attendanceDayVO != null) {
                attendanceDays.add(attendanceDayVO);
            }
            // -- 考勤工时计算
            AttendancePunchStatisticsVO averageWorkingHourVO = this.calculateAverageWorkingHour(day, weekChinese, workMap, calculateNum);
            averageWorkingHours.add(averageWorkingHourVO);

            // -- 迟到计算
            List<AttendancePunchStatisticsVO> beLateList = this.calculateBeLate(day, weekChinese, workMap, calculateNum);
            beLateNumber.addAll(beLateList);
            // -- 早退计算
            List<AttendancePunchStatisticsVO> earlyDeparturesList = this.calculateEarlyDepartures(day, weekChinese, workMap, calculateNum);
            earlyDeparturesNumber.addAll(earlyDeparturesList);

            // -- 缺卡计算
            List<AttendancePunchStatisticsVO> missingCardsList = this.calculateMissingCardsList(day, weekChinese, workMap, calculateNum);
            missingCardsNumber.addAll(missingCardsList);

            // -- 旷工计算
            AttendancePunchStatisticsVO miner = this.calculateMiner(day, weekChinese, entry.getValue(), calculateNum);
            if (miner != null) {
                minerDays.add(miner);
            }
            // -- 外勤计算
            List<AttendancePunchStatisticsVO> fieldServiceList = this.calculateFieldService(day, weekChinese, workMap, calculateNum);
            fieldServiceNumber.addAll(fieldServiceList);
        }
        // -- 休息计算
        List<AttendancePunchStatisticsVO> restDays = this.calculateRestDayList(dateList, map.keySet().stream().distinct().collect(Collectors.toList()), calculateNum);
        calculateNum.setAverageWorkingHours(calculateNum.getTotalAttendanceDays() == 0 ? 0 : calculateNum.getTotalWorkingHours() / calculateNum.getTotalAttendanceDays());
        calculateNum.setAverageWorkingHoursStr(DateUtil.formatBetween(calculateNum.getAverageWorkingHours(), BetweenFormatter.Level.MINUTE));
        calculateNum.setTotalLateArrivalsTimeStr(DateUtil.formatBetween(calculateNum.getTotalLateArrivalsTime(), BetweenFormatter.Level.MINUTE));
        calculateNum.setTotalEarlyDeparturesTimeStr(DateUtil.formatBetween(calculateNum.getTotalEarlyDeparturesTime(), BetweenFormatter.Level.MINUTE));
        vo.setAttendanceDays(attendanceDays);
        vo.setAverageWorkingHours(averageWorkingHours);
        vo.setBeLateNumber(beLateNumber);
        vo.setEarlyDeparturesNumber(earlyDeparturesNumber);
        vo.setMissingCardsNumber(missingCardsNumber);
        vo.setMinerDays(minerDays);
        vo.setFieldServiceNumber(fieldServiceNumber);
        vo.setRestDays(restDays);
        vo.setCalculateNum(calculateNum);
        return vo;
    }

    @Override
    public Map<String, TeamAttendanceStatisticsByDayVO> teamStatisticsByDay(TeamAttendanceStatisticsByDayDTO dto) {
        Map<String, TeamAttendanceStatisticsByDayVO> map = new HashMap<>();
        //查询考勤组
        AttendanceGroupDO attendanceGroupDO = attendanceGroupService.getGroup(dto.getGroupId());
        // - 判断当前用户是否有权限查看
        // TODO: 2024/5/28 这里需要根据数据权限再过滤一下 ？ 或者说移动端不让看 有数据权限的直接在管理后台看
        if (!dto.getUserId().equals(attendanceGroupDO.getUserId())) {
            throw exception(NO_PERMISSION_TO_VIEW_CURRENT_ATTENDANCE_GROUP_INFORMATION);
        }
        List<String> dateList = DateUtils.betweenDayList(DateUtil.beginOfMonth(dto.getTime()),
                DateUtil.endOfMonth(dto.getTime()));
        List<AttendancePunchRecordDO> list = attendancePunchRecordMapper.selectList(new LambdaQueryWrapper<AttendancePunchRecordDO>()
                .eq(AttendancePunchRecordDO::getAttendanceGroupId, dto.getGroupId())
                .in(AttendancePunchRecordDO::getDayTime, dateList)
                .orderByAsc(AttendancePunchRecordDO::getDayTime)
                .orderByAsc(AttendancePunchRecordDO::getLevel)
                .orderByAsc(AttendancePunchRecordDO::getWorkType));
        Map<String, List<AttendancePunchRecordDO>> punchRecorMap = list.stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getDayTime));

        for (Map.Entry<String, List<AttendancePunchRecordDO>> entry : punchRecorMap.entrySet()) {
            Map<Long, List<AttendancePunchRecordDO>> userMap = entry.getValue().stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getUserId));
            Map<Long, List<AttendancePunchRecordDO>> punchMap = entry.getValue().stream().filter(a -> ObjectUtil.isNotEmpty(a.getPunchTime())).collect(Collectors.groupingBy(AttendancePunchRecordDO::getUserId));
            int punchStatusMissNum = 0;
            int lateNum = 0;
            int leaveEarlyNum = 0;
            int fieldworkNum = 0;
            for (AttendancePunchRecordDO attendancePunchRecordDO : entry.getValue()) {
                if (AttendanceOnTheDayDTO.PUNCH_STATUS_MISS.equals(attendancePunchRecordDO.getStatus())) {
                    punchStatusMissNum++;
                }
                if (AttendanceOnTheDayDTO.PUNCH_STATUS_LEAVE_EARLY.equals(attendancePunchRecordDO.getStatus())) {
                    lateNum++;
                }
                if (AttendanceOnTheDayDTO.PUNCH_STATUS_LATE.equals(attendancePunchRecordDO.getStatus())) {
                    leaveEarlyNum++;
                }
                if (Constants.TRUE.equals(attendancePunchRecordDO.getFieldServiceFlag())) {
                    fieldworkNum++;
                }
            }
            TeamAttendanceStatisticsByDayVO vo = new TeamAttendanceStatisticsByDayVO();
            vo.setAnswerNum(userMap.keySet().size());
            vo.setPunchNum(punchMap.keySet().size());
            vo.setPunchStatusMissNum(punchStatusMissNum);
            vo.setLateNum(lateNum);
            vo.setLeaveEarlyNum(leaveEarlyNum);
            vo.setFieldworkNum(fieldworkNum);
            map.put(entry.getKey(), vo);
        }
        return map;
    }

    @Override
    public TeamAttendanceStatisticsByCycleVO tesmStatisticsByCycle(TeamAttendanceStatisticsByCycleDTO dto) {
        TeamAttendanceStatisticsByCycleVO vo = new TeamAttendanceStatisticsByCycleVO();
        TeamAttendanceStatisticsByCycleVO.TeamAttendanceStatisticsNumVO teamAttendanceStatisticsNumVO = new TeamAttendanceStatisticsByCycleVO.TeamAttendanceStatisticsNumVO();
        //查询考勤组
        AttendanceGroupDO attendanceGroupDO = attendanceGroupService.getGroup(dto.getGroupId());
        // - 判断当前用户是否有权限查看
        // TODO: 2024/5/28 这里需要根据数据权限再过滤一下 ？ 或者说移动端不让看 有数据权限的直接在管理后台看
        if (!dto.getUserId().equals(attendanceGroupDO.getUserId())) {
            throw exception(NO_PERMISSION_TO_VIEW_CURRENT_ATTENDANCE_GROUP_INFORMATION);
        }
        Date thisTime = new Date();
        Date beginTime = dto.getStartTime();
        Date endTime = dto.getEndTime();
        if (beginTime.getTime() > endTime.getTime()) {
            throw exception(THE_START_TIME_CANNOT_BE_GREATER_THAN_THE_END_TIME);
        }
        // -- 如果结束时间大于当前时间 - 那么结束时间 = 当前时间
        if (endTime.getTime() > thisTime.getTime()) {
            endTime = thisTime;
        }
        List<String> dateList = DateUtils.betweenDayList(beginTime, endTime);
        List<AttendancePunchRecordDO> list = attendancePunchRecordMapper.selectList(new LambdaQueryWrapper<AttendancePunchRecordDO>()
                .eq(AttendancePunchRecordDO::getAttendanceGroupId, dto.getGroupId())
                .in(AttendancePunchRecordDO::getDayTime, dateList)
                .orderByAsc(AttendancePunchRecordDO::getDayTime)
                .orderByAsc(AttendancePunchRecordDO::getLevel)
                .orderByAsc(AttendancePunchRecordDO::getWorkType));


        //获取用户列表
        List<Long> userIds = list.stream().map(AttendancePunchRecordDO::getUserId).distinct().collect(Collectors.toList());
        List<AdminUserDO> userList = adminUserService.getUserList(userIds);
        Map<Long, AdminUserDO> userMap = new HashMap<>();
        if (CollectionUtil.isNotEmpty(userList)) {
            userMap = userList.stream().collect(Collectors.toMap(AdminUserDO::getId, Function.identity()));
        }
        //平均工时
        List<TeamAttendancePunchStatisticsVO> averageWorkingHours = new ArrayList<>();
        //迟到
        List<TeamAttendancePunchStatisticsVO> beLateList = new ArrayList<>();
        //早退
        List<TeamAttendancePunchStatisticsVO> leaveEarlyList = new ArrayList<>();
        //缺卡
        List<TeamAttendancePunchStatisticsVO> missingCardList = new ArrayList<>();
        //旷工
        List<TeamAttendancePunchStatisticsVO> absenteeismList = new ArrayList<>();
        //外勤
        List<TeamAttendancePunchStatisticsVO> fieldServiceList = new ArrayList<>();

        Map<Long, List<AttendancePunchRecordDO>> userPunchMap = list.stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getUserId));
        // -- 根据用户分组
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> entry : userPunchMap.entrySet()) {
            // 通用头部 ------------------------------------
            AdminUserDO adminUserDO = userMap.get(entry.getKey());
            TeamAttendancePunchStatisticsVO averageWorkingHourVO = new TeamAttendancePunchStatisticsVO();
            averageWorkingHourVO.setUserId(entry.getKey());
            if (adminUserDO != null) {
                averageWorkingHourVO.setName(adminUserDO.getNickname());
                averageWorkingHourVO.setAvatar(adminUserDO.getAvatar());
            }
            // 通用头部 ------------------------------------

            CalculateNum calculateNum = new CalculateNum();
            //按日期分组
            Map<String, List<AttendancePunchRecordDO>> dayMap = entry.getValue().stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getDayTime));
            for (Map.Entry<String, List<AttendancePunchRecordDO>> dayEntry : dayMap.entrySet()) {
                // -- 按照班次子表分组
                Map<Long, List<AttendancePunchRecordDO>> workMap = dayEntry.getValue().stream().collect(Collectors.groupingBy(AttendancePunchRecordDO::getAttendanceGroupShiftItemId));
                this.calculateAverageWorkingHour(workMap, calculateNum);
                // -- 出考勤天数
                this.calculateAttendanceDays(dayEntry.getValue(), calculateNum);
                // -- 迟到
                this.calculateBeLate(workMap, calculateNum);
                // -- 早退
                this.calculateEarlyDepartures(workMap, calculateNum);
                // -- 缺卡
                this.calculateMissingCardsList(workMap, calculateNum);
                // -- 旷工
                this.calculateMiner(dayEntry.getValue(), calculateNum);
                // -- 外勤
                this.calculateFieldService(workMap, calculateNum);
            }
            calculateNum.setAverageWorkingHours(calculateNum.getTotalAttendanceDays() == 0 ? 0 : calculateNum.getTotalWorkingHours() / calculateNum.getTotalAttendanceDays());
            calculateNum.setAverageWorkingHoursStr(DateUtil.formatBetween(calculateNum.getAverageWorkingHours(), BetweenFormatter.Level.MINUTE));
            calculateNum.setTotalLateArrivalsTimeStr(DateUtil.formatBetween(calculateNum.getTotalLateArrivalsTime(), BetweenFormatter.Level.MINUTE));
            calculateNum.setTotalEarlyDeparturesTimeStr(DateUtil.formatBetween(calculateNum.getTotalEarlyDeparturesTime(), BetweenFormatter.Level.MINUTE));
            averageWorkingHourVO.setCalculateNum(calculateNum);

            // --- 平均工时
            averageWorkingHours.add(averageWorkingHourVO.setTop(calculateNum.getAverageWorkingHoursStr()).setDown("出勤" + calculateNum.getTotalAttendanceDays() + "天"));
            // --- 迟到
            if (calculateNum.getTotalLateArrivalsNumber() > 0) {
                TeamAttendancePunchStatisticsVO item = new TeamAttendancePunchStatisticsVO();
                BeanUtil.copyProperties(averageWorkingHourVO, item);
                beLateList.add(item.setTop(calculateNum.getTotalLateArrivalsNumber() + "次").setDown(calculateNum.getTotalLateArrivalsTimeStr()));
            }
            // --- 早退
            if (calculateNum.getTotalEarlyDeparturesNumber() > 0) {
                TeamAttendancePunchStatisticsVO item = new TeamAttendancePunchStatisticsVO();
                BeanUtil.copyProperties(averageWorkingHourVO, item);
                leaveEarlyList.add(item.setTop(calculateNum.getTotalEarlyDeparturesNumber() + "次").setDown(calculateNum.getTotalEarlyDeparturesTimeStr()));
            }
            // --- 缺卡
            if (calculateNum.getTotalMissingCardsNumber() > 0) {
                TeamAttendancePunchStatisticsVO item = new TeamAttendancePunchStatisticsVO();
                BeanUtil.copyProperties(averageWorkingHourVO, item);
                missingCardList.add(item.setTop(calculateNum.getTotalMissingCardsNumber() + "次").setDown(""));
            }
            // --- 旷工
            if (calculateNum.getTotalMinerDays() > 0) {
                TeamAttendancePunchStatisticsVO item = new TeamAttendancePunchStatisticsVO();
                BeanUtil.copyProperties(averageWorkingHourVO, item);
                absenteeismList.add(item.setTop(calculateNum.getTotalMinerDays() + "天").setDown(""));
            }
            // --- 外勤
            if (calculateNum.getTotalFieldServiceNumber() > 0) {
                TeamAttendancePunchStatisticsVO item = new TeamAttendancePunchStatisticsVO();
                BeanUtil.copyProperties(averageWorkingHourVO, item);
                fieldServiceList.add(item.setTop(calculateNum.getTotalFieldServiceNumber() + "次").setDown(""));
            }
        }

        vo.setAverageWorkingHoursList(averageWorkingHours);
        vo.setBeLateList(beLateList);
        vo.setLeaveEarlyList(leaveEarlyList);
        vo.setMissingCardList(missingCardList);
        vo.setAbsenteeismList(absenteeismList);
        vo.setFieldServiceList(fieldServiceList);
        int sum = averageWorkingHours.stream().mapToInt(a -> a.getCalculateNum().getTotalAttendanceDays()).sum();
        TeamAttendanceStatisticsByCycleVO.TeamAttendanceStatisticsNumVO teamAttendanceStatisticsByCycleVO = new TeamAttendanceStatisticsByCycleVO.TeamAttendanceStatisticsNumVO();
        teamAttendanceStatisticsByCycleVO.setAverageWorkingHours(
                sum == 0 ? 0 : averageWorkingHours.stream().mapToLong(a -> a.getCalculateNum().getTotalWorkingHours()).sum() / sum
        );
        vo.setTeamAttendanceStatisticsNumVO(teamAttendanceStatisticsByCycleVO
                .setBeLateNum(beLateList.stream().mapToInt(a -> a.getCalculateNum().getTotalLateArrivalsNumber()).sum())
                .setLeaveEarlyNum(leaveEarlyList.stream().mapToInt(a -> a.getCalculateNum().getTotalEarlyDeparturesNumber()).sum())
                .setMissingCardNum(missingCardList.stream().mapToInt(a -> a.getCalculateNum().getTotalMissingCardsNumber()).sum())
                .setAbsenteeismNum(absenteeismList.stream().mapToInt(a -> a.getCalculateNum().getTotalMinerDays()).sum())
                .setFieldServiceNum(fieldServiceList.stream().mapToInt(a -> a.getCalculateNum().getTotalFieldServiceNumber()).sum())
        );
        return vo;
    }

    @Override
    public List<AttendanceGroupSystemVO> getTheAttendanceGroupToWhichTheCurrentlyLoggedInUserBelongsAndWhetherTheyHaveAdministratorRights() {
        Long userId = getLoginUserId();
        List<AttendanceGroupSystemVO> attendanceGroupSystemVOS = new ArrayList<>();
        List<AttendanceGroupDO> attendanceGroupDOS = attendanceGroupMapper.selectList(new LambdaQueryWrapper<AttendanceGroupDO>()
                .eq(AttendanceGroupDO::getUserId, userId));
        if (CollectionUtil.isEmpty(attendanceGroupDOS)) {
            return attendanceGroupSystemVOS;
        }
        for (AttendanceGroupDO attendanceGroupDO : attendanceGroupDOS) {
            AttendanceGroupSystemVO attendanceGroupSystemVO = new AttendanceGroupSystemVO();
            attendanceGroupSystemVO.setGroupId(attendanceGroupDO.getId());
            attendanceGroupSystemVO.setGroupName(attendanceGroupDO.getGroupName());
            attendanceGroupSystemVOS.add(attendanceGroupSystemVO);
        }
        return attendanceGroupSystemVOS;
    }

    @Override
    public AttendanceRulesVO getRules(Long groupId) {
        AttendanceRulesVO vo = new AttendanceRulesVO();
        AttendanceGroupDO group = attendanceGroupService.getGroup(groupId);
        PunchService punchService = punchHandler.getResource(AttendanceGroupDO.getCodeByType(group.getType()));
        List<AttendanceRulesVO.WorkRules> workRules = punchService.getRule(groupId);
        vo.setFieldworkFlag(group.getFieldworkFlag());
        vo.setWorkRules(workRules);
        vo.setOfficeLocation(group.getAddress());
        return vo;
    }

    @Override
    public void exportAttendanceExcel(HttpServletResponse response, ExportAttendanceExcelDTO dto) {
        List<AdminUserDO> userList = new ArrayList<>();
        List<Long> userIds = new ArrayList<>();
        if (dto.getMemberRange() == 1) {
            userList = adminUserService.getAllList(CommonStatusEnum.ENABLE.getStatus(), null, userIds);
        } else if (dto.getMemberRange() == 2) {
            // -- 获取考勤组ids
            userIds = attendanceGroupUserService.getUserIdsByGroupIds(dto.getTargetIds());
            userList = adminUserService.getAllList(CommonStatusEnum.ENABLE.getStatus(), null, userIds);
        } else {
            userList = adminUserService.getAllList(CommonStatusEnum.ENABLE.getStatus(), null, dto.getTargetIds());
        }
        // -- 统计
        List<String> dateList = DateUtils.betweenDayList(dto.getStartTime(), dto.getEndTime());
        if (dto.getType() == 1) {
            this.monthlyStatistics(response, userList, dateList);
        }
    }

    /**
     * 按月导出
     *
     * @param response
     * @param userList
     * @param dateList
     */
    private void monthlyStatistics(HttpServletResponse response, List<AdminUserDO> userList, List<String> dateList) {
        //查询出数据 -- 时间周期 - 用户列表
        List<AttendancePunchRecordDO> list = attendancePunchRecordMapper.selectList(new LambdaQueryWrapper<AttendancePunchRecordDO>()
                .in(AttendancePunchRecordDO::getUserId, userList)
                .in(AttendancePunchRecordDO::getDayTime, dateList));
        // -- 根据部门分组 - 根据考勤组分组


    }

    public static void main(String[] args) {
        String fileName = "/Users/aikai/Downloads/" + System.currentTimeMillis() + ".xlsx";
        // 这里 需要指定写用哪个class去写，然后写到第一个sheet，名字为模板 然后文件流会自动关闭
        EasyExcel.write(fileName)
                .head(generateHead())
                .registerWriteHandler(new CustomCellStyleHandler())
//                .registerWriteHandler(new LoopMergeStrategy())
//                .registerWriteHandler(loopMergeStrategy)
                .sheet("模板")
                .doWrite(generateData());
    }

    public static List<List<String>> generateHead() {
        List<List<String>> head = new ArrayList<>();
        String headTitle = "月度汇总 统计日期：2024-06-01 至 2024-06-13";
        String detailedHead = "月度汇总 统计日期：2024-06-01 至 2024-06-13 10:48";
        head.add(Arrays.asList(headTitle, detailedHead, "姓名", "姓名"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤组", "考勤组"));
        head.add(Arrays.asList(headTitle, detailedHead, "部门", "部门"));
        head.add(Arrays.asList(headTitle, detailedHead, "工号", "工号"));
        head.add(Arrays.asList(headTitle, detailedHead, "职位", "职位"));
        head.add(Arrays.asList(headTitle, detailedHead, "出勤天数", "出勤天数"));
        head.add(Arrays.asList(headTitle, detailedHead, "休息天数", "休息天数"));
        head.add(Arrays.asList(headTitle, detailedHead, "工作时长", "工作时长"));
        head.add(Arrays.asList(headTitle, detailedHead, "迟到次数", "迟到次数"));
        head.add(Arrays.asList(headTitle, detailedHead, "迟到时长", "迟到时长"));
        head.add(Arrays.asList(headTitle, detailedHead, "早退次数", "早退次数"));
        head.add(Arrays.asList(headTitle, detailedHead, "早退时长", "早退时长"));
        head.add(Arrays.asList(headTitle, detailedHead, "上班缺卡次数", "上班缺卡次数"));
        head.add(Arrays.asList(headTitle, detailedHead, "下班缺卡次数", "下班缺卡次数"));
        head.add(Arrays.asList(headTitle, detailedHead, "旷工天数", "旷工天数"));
//        head.add(Arrays.asList(headTitle, detailedHead, "出差时长", "出差时长"));
//        head.add(Arrays.asList(headTitle, detailedHead, "外出时长", "外出时长"));
//        head.add(Arrays.asList(headTitle, detailedHead, "加班-审批单统计", "加班-审批单统计"));
//        head.add(Arrays.asList(headTitle, detailedHead, "加班时长-按加班规则计算", "工作日加班"));
//        head.add(Arrays.asList(headTitle, detailedHead, "加班时长-按加班规则计算", "休息日加班"));
//        head.add(Arrays.asList(headTitle, detailedHead, "加班时长-按加班规则计算", "节假日加班"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "六"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "日"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "1"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "2"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "3"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "4"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "5"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "六"));
        head.add(Arrays.asList(headTitle, detailedHead, "考勤结果", "日"));
        // 添加更多表头
        return head;
    }

    public static List<List<Object>> generateData() {
        List<List<Object>> data = new ArrayList<>();

        List<Object> row1 = new ArrayList<>();
        row1.add("艾楷");
        row1.add("测试考勤");
        row1.add("");
        row1.add("");
        row1.add("");
        row1.add("22026829221062585");
        row1.add(1);
        row1.add(2);
        row1.add(3);
        row1.add(4);
        row1.add(5);
        row1.add(6);
        row1.add(7);
        row1.add(8);
        row1.add(9);
        row1.add(10);
        row1.add(10);
        row1.add(12);
        // 添加更多数据

        List<Object> row2 = new ArrayList<>();
        row2.add("谢鸿飞");
        row2.add("测试考勤");
        row2.add("");
        row2.add("");
        row2.add("");
        row2.add("066557433135769889");
        row2.add(1);
        row2.add(2);
        row2.add(3);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        row2.add(4);
        // 添加更多数据

        data.add(row1);
        data.add(row2);

        return data;
    }

    /**
     * 外勤计算
     *
     * @param day
     * @param weekChinese
     * @param workMap
     * @param calculateNum
     * @return
     */
    private List<AttendancePunchStatisticsVO> calculateFieldService(String day, String weekChinese, Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        List<AttendancePunchStatisticsVO> list = new ArrayList<>();
        int todayFieldService = 0;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 缺卡计算
                List<AttendancePunchRecordDO> fieldServiceList = workEntry.getValue().stream().filter(a -> Constants.TRUE.equals(a.getFieldServiceFlag())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(fieldServiceList)) {
                    todayFieldService += fieldServiceList.size();
                    for (AttendancePunchRecordDO attendancePunchRecordDO : fieldServiceList) {
                        AttendancePunchStatisticsVO vo = new AttendancePunchStatisticsVO();
                        vo.setDay(day);
                        vo.setWeek(weekChinese);
                        vo.setShouldPunchTime(attendancePunchRecordDO.getShouldPunchTime().format(DateTimeFormatter.ofPattern("HH:mm")));
                        list.add(vo);
                    }
                }
            }
        }
        calculateNum.setTotalFieldServiceNumber(calculateNum.getTotalFieldServiceNumber() + todayFieldService);
        return list;
    }


    private void calculateFieldService(Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        int todayFieldService = 0;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 缺卡计算
                List<AttendancePunchRecordDO> fieldServiceList = workEntry.getValue().stream().filter(a -> Constants.TRUE.equals(a.getFieldServiceFlag())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(fieldServiceList)) {
                    todayFieldService += fieldServiceList.size();
                }
            }
        }
        calculateNum.setTotalFieldServiceNumber(calculateNum.getTotalFieldServiceNumber() + todayFieldService);
    }

    /**
     * 计算矿工
     *
     * @param day
     * @param weekChinese
     * @param calculateNum
     * @return
     */
    private AttendancePunchStatisticsVO calculateMiner(String day, String weekChinese, List<AttendancePunchRecordDO> list, CalculateNum calculateNum) {
        AttendancePunchStatisticsVO vo = null;
        List<AttendancePunchRecordDO> collect = list.stream().filter(a -> a.getStatus().equals(AttendanceOnTheDayDTO.PUNCH_STATUS_MISS)).collect(Collectors.toList());
        if (collect.size() == list.size()) {
            vo = new AttendancePunchStatisticsVO();
            vo.setDay(day);
            vo.setWeek(weekChinese);
            calculateNum.setTotalMinerDays(calculateNum.getTotalMinerDays() + 1);
        }
        return vo;
    }

    private void calculateMiner(List<AttendancePunchRecordDO> list, CalculateNum calculateNum) {
        List<AttendancePunchRecordDO> collect = list.stream().filter(a -> a.getStatus().equals(AttendanceOnTheDayDTO.PUNCH_STATUS_MISS)).collect(Collectors.toList());
        if (collect.size() == list.size()) {
            calculateNum.setTotalMinerDays(calculateNum.getTotalMinerDays() + 1);
        }
    }

    /**
     * 休息时间计算
     *
     * @param dateList
     * @param attendanceTime
     * @param calculateNum
     * @return
     */
    private List<AttendancePunchStatisticsVO> calculateRestDayList(List<String> dateList, List<String> attendanceTime, CalculateNum calculateNum) {
        List<AttendancePunchStatisticsVO> list = new ArrayList<>();
        List<String> subtract = new ArrayList<>(CollectionUtil.subtract(dateList, attendanceTime));
        for (String timeStr : subtract) {
            AttendancePunchStatisticsVO vo = new AttendancePunchStatisticsVO();
            vo.setDay(timeStr);
            vo.setWeek(dayOfWeekEnum(DateUtil.parse(timeStr)).toChinese());
            list.add(vo);
        }
        calculateNum.setTotalRestDays(list.size());
        return list;
    }


    /**
     * @param day
     * @param weekChinese
     * @param workMap
     * @param calculateNum
     * @return
     */
    private List<AttendancePunchStatisticsVO> calculateMissingCardsList(String day, String weekChinese, Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        List<AttendancePunchStatisticsVO> list = new ArrayList<>();
        int todayMissingCardsNumber = 0;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 缺卡计算
                List<AttendancePunchRecordDO> missingCardsList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_MISS.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(missingCardsList)) {
                    todayMissingCardsNumber += missingCardsList.size();
                    for (AttendancePunchRecordDO attendancePunchRecordDO : missingCardsList) {
                        AttendancePunchStatisticsVO earlyDepartures = new AttendancePunchStatisticsVO();
                        earlyDepartures.setDay(day);
                        earlyDepartures.setWeek(weekChinese);
                        earlyDepartures.setShouldPunchTime(attendancePunchRecordDO.getShouldPunchTime().format(DateTimeFormatter.ofPattern("HH:mm")));
                        list.add(earlyDepartures);
                    }
                }
            }
        }
        calculateNum.setTotalMissingCardsNumber(calculateNum.getTotalMissingCardsNumber() + todayMissingCardsNumber);
        return list;
    }

    private void calculateMissingCardsList(Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        int todayMissingCardsNumber = 0;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 缺卡计算
                List<AttendancePunchRecordDO> missingCardsList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_MISS.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(missingCardsList)) {
                    todayMissingCardsNumber += missingCardsList.size();
                }
            }
        }
        calculateNum.setTotalMissingCardsNumber(calculateNum.getTotalMissingCardsNumber() + todayMissingCardsNumber);
    }

    /**
     * 早退计算
     *
     * @param day
     * @param weekChinese
     * @param workMap
     * @param calculateNum
     * @return
     */
    private List<AttendancePunchStatisticsVO> calculateEarlyDepartures(String day, String weekChinese, Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        List<AttendancePunchStatisticsVO> list = new ArrayList<>();
        int todayEarlyDeparturesNumber = 0;
        long todayEarlyDeparturesTime = 0L;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 早退计算
                List<AttendancePunchRecordDO> earlyDeparturesList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_LEAVE_EARLY.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(earlyDeparturesList)) {
                    todayEarlyDeparturesNumber += earlyDeparturesList.size();
                    for (AttendancePunchRecordDO attendancePunchRecordDO : earlyDeparturesList) {
                        AttendancePunchStatisticsVO earlyDepartures = new AttendancePunchStatisticsVO();
                        earlyDepartures.setDay(day);
                        earlyDepartures.setWeek(weekChinese);
                        earlyDepartures.setShouldPunchTime(attendancePunchRecordDO.getShouldPunchTime().format(DateTimeFormatter.ofPattern("HH:mm")));
                        earlyDepartures.setTime("下班早退" + DateUtil.formatBetween(attendancePunchRecordDO.getLeaveEarlyTime(), BetweenFormatter.Level.MINUTE));
                        list.add(earlyDepartures);
                    }
                    todayEarlyDeparturesTime += earlyDeparturesList.stream().mapToLong(AttendancePunchRecordDO::getLeaveEarlyTime).sum();
                }
            }
        }
        calculateNum.setTotalEarlyDeparturesNumber(calculateNum.getTotalEarlyDeparturesNumber() + todayEarlyDeparturesNumber);
        calculateNum.setTotalEarlyDeparturesTime(calculateNum.getTotalEarlyDeparturesTime() + todayEarlyDeparturesTime);
        return list;
    }


    private void calculateEarlyDepartures(Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        int todayEarlyDeparturesNumber = 0;
        long todayEarlyDeparturesTime = 0L;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 早退计算
                List<AttendancePunchRecordDO> earlyDeparturesList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_LEAVE_EARLY.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(earlyDeparturesList)) {
                    todayEarlyDeparturesNumber += earlyDeparturesList.size();
                    todayEarlyDeparturesTime += earlyDeparturesList.stream().mapToLong(AttendancePunchRecordDO::getLeaveEarlyTime).sum();
                }
            }
        }
        calculateNum.setTotalEarlyDeparturesNumber(calculateNum.getTotalEarlyDeparturesNumber() + todayEarlyDeparturesNumber);
        calculateNum.setTotalEarlyDeparturesTime(calculateNum.getTotalEarlyDeparturesTime() + todayEarlyDeparturesTime);
    }

    /**
     * 计算迟到
     *
     * @param day
     * @param weekChinese
     * @param workMap
     * @param calculateNum
     * @return
     */
    private List<AttendancePunchStatisticsVO> calculateBeLate(String day, String weekChinese, Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        List<AttendancePunchStatisticsVO> list = new ArrayList<>();
        int todayLateArrivalsNumber = 0;
        long todayLateArrivalsTime = 0L;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 迟到计算
                List<AttendancePunchRecordDO> beLateList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_LATE.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(beLateList)) {
                    todayLateArrivalsNumber += beLateList.size();
                    for (AttendancePunchRecordDO attendancePunchRecordDO : beLateList) {
                        AttendancePunchStatisticsVO beLateDay = new AttendancePunchStatisticsVO();
                        beLateDay.setDay(day);
                        beLateDay.setWeek(weekChinese);
                        beLateDay.setShouldPunchTime(attendancePunchRecordDO.getShouldPunchTime().format(DateTimeFormatter.ofPattern("HH:mm")));
                        beLateDay.setTime("上班迟到" + DateUtil.formatBetween(attendancePunchRecordDO.getLateTime(), BetweenFormatter.Level.MINUTE));
                        //将时间戳lateTime 转换成 **小时**分格式 不满1小时的时间不显小时
                        list.add(beLateDay);
                    }
                    todayLateArrivalsTime += beLateList.stream().mapToLong(AttendancePunchRecordDO::getLateTime).sum();
                }
            }
        }
        calculateNum.setTotalLateArrivalsNumber(calculateNum.getTotalLateArrivalsNumber() + todayLateArrivalsNumber);
        calculateNum.setTotalLateArrivalsTime(calculateNum.getTotalLateArrivalsTime() + todayLateArrivalsTime);
        return list;
    }


    private void calculateBeLate(Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        int todayLateArrivalsNumber = 0;
        long todayLateArrivalsTime = 0L;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 迟到计算
                List<AttendancePunchRecordDO> beLateList = workEntry.getValue().stream().filter(a -> AttendanceOnTheDayDTO.PUNCH_STATUS_LATE.equals(a.getStatus())).collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(beLateList)) {
                    todayLateArrivalsNumber += beLateList.size();
                    todayLateArrivalsTime += beLateList.stream().mapToLong(AttendancePunchRecordDO::getLateTime).sum();
                }
            }
        }
        calculateNum.setTotalLateArrivalsNumber(calculateNum.getTotalLateArrivalsNumber() + todayLateArrivalsNumber);
        calculateNum.setTotalLateArrivalsTime(calculateNum.getTotalLateArrivalsTime() + todayLateArrivalsTime);
    }

    /**
     * 计算考勤工时
     *
     * @param day
     * @param weekChinese
     * @param workMap
     * @param calculateNum
     * @return
     */
    private AttendancePunchStatisticsVO calculateAverageWorkingHour(String day, String weekChinese, Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        AttendancePunchStatisticsVO vo = new AttendancePunchStatisticsVO();
        long todayWorkingHours = this.calculateAverageWorkingHour(workMap, calculateNum);
        vo.setDay(day);
        vo.setWeek(weekChinese);
        vo.setTime(DateUtil.formatBetween(todayWorkingHours, BetweenFormatter.Level.MINUTE));
        return vo;
    }

    /**
     * 计算考勤工时
     *
     * @param workMap
     * @return
     */
    private long calculateAverageWorkingHour(Map<Long, List<AttendancePunchRecordDO>> workMap, CalculateNum calculateNum) {
        long todayWorkingHours = 0L;
        for (Map.Entry<Long, List<AttendancePunchRecordDO>> workEntry : workMap.entrySet()) {
            if (CollectionUtil.isNotEmpty(workEntry.getValue())) {
                // -- 工时计算 - 有上班和下班两个卡 才计算到工时里面
                if (workEntry.getValue().size() >= 2) {
                    AttendancePunchRecordDO first = CollectionUtil.getFirst(workEntry.getValue());
                    AttendancePunchRecordDO last = CollectionUtil.getLast(workEntry.getValue());
                    if (ObjectUtil.isNotEmpty(first.getPunchTime()) && ObjectUtil.isNotEmpty(last.getPunchTime())) {
                        long time = Math.abs(LocalDateTimeUtil.between(first.getPunchTime(), last.getPunchTime(), ChronoUnit.MILLIS));
                        todayWorkingHours += time;
                    }
                }
            }
        }
        calculateNum.setTotalWorkingHours(calculateNum.getTotalWorkingHours() + todayWorkingHours);
        return todayWorkingHours;
    }


    /**
     * 计算出勤天数
     *
     * @param day
     * @param weekChinese
     * @param list
     * @param calculateNum
     * @return
     */
    private AttendancePunchStatisticsVO calculateAttendanceDays(String day, String weekChinese, List<AttendancePunchRecordDO> list, CalculateNum calculateNum) {
        AttendancePunchStatisticsVO averageWorkingHourVO = null;
        List<AttendancePunchRecordDO> collect = list.stream().filter(a -> ObjectUtil.isNotEmpty(a.getPunchTime())).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(collect)) {
            averageWorkingHourVO = new AttendancePunchStatisticsVO();
            averageWorkingHourVO.setDay(day);
            averageWorkingHourVO.setWeek(weekChinese);
            calculateNum.setTotalAttendanceDays(calculateNum.getTotalAttendanceDays() + 1);
        }
        return averageWorkingHourVO;
    }

    private void calculateAttendanceDays(List<AttendancePunchRecordDO> list, CalculateNum calculateNum) {
        List<AttendancePunchRecordDO> collect = list.stream().filter(a -> ObjectUtil.isNotEmpty(a.getPunchTime())).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(collect)) {
            calculateNum.setTotalAttendanceDays(calculateNum.getTotalAttendanceDays() + 1);
        }
    }

}






















